"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOpenApiMetadata = void 0;
const lodash_zipobject_1 = __importDefault(require("lodash.zipobject"));
const path_to_regexp_1 = require("path-to-regexp");
const openapi_core_1 = require("openapi-core");
const util_1 = require("./util");
function applyOpenApiMetadata(openApiContext, responseApiDoc) {
    return (req, res, next) => {
        const path = util_1.pathname(req);
        if (openApiContext.shouldIgnoreRoute(path)) {
            return next();
        }
        const matched = lookupRoute(req);
        if (matched) {
            const { expressRoute, openApiRoute, pathParams, schema } = matched;
            if (!schema) {
                throw new openapi_core_1.MethodNotAllowed({
                    path: req.path,
                    message: `${req.method} method not allowed`,
                });
            }
            req.openapi = {
                expressRoute: expressRoute,
                openApiRoute: openApiRoute,
                pathParams: pathParams,
                schema: schema,
            };
            req.params = pathParams;
            if (responseApiDoc) {
                // add the response schema if validating responses
                req.openapi._responseSchema = matched._responseSchema;
            }
        }
        else if (openApiContext.isManagedRoute(path)) {
            throw new openapi_core_1.NotFound({
                path: req.path,
                message: 'not found',
            });
        }
        next();
    };
    function lookupRoute(req) {
        var _a, _b, _c;
        const path = ((_a = req.originalUrl) !== null && _a !== void 0 ? _a : req.url).split('?')[0];
        const method = req.method;
        const routeEntries = Object.entries(openApiContext.expressRouteMap);
        for (const [expressRoute, methods] of routeEntries) {
            const routePair = openApiContext.routePair(expressRoute);
            const openApiRoute = routePair.openApiRoute;
            const pathKey = openApiRoute.substring(methods.basePath.length);
            const schema = openApiContext.apiDoc.paths[pathKey][method.toLowerCase()];
            const _schema = responseApiDoc === null || responseApiDoc === void 0 ? void 0 : responseApiDoc.paths[pathKey][method.toLowerCase()];
            const keys = [];
            const strict = !!((_b = req.app) === null || _b === void 0 ? void 0 : _b.enabled('strict routing'));
            const sensitive = !!((_c = req.app) === null || _c === void 0 ? void 0 : _c.enabled('case sensitive routing'));
            const pathOpts = {
                sensitive,
                strict,
            };
            const regexp = path_to_regexp_1.pathToRegexp(expressRoute, keys, pathOpts);
            const matchedRoute = regexp.exec(path);
            if (matchedRoute) {
                const paramKeys = keys.map((k) => k.name);
                const paramsVals = matchedRoute.slice(1).map(decodeURIComponent);
                const pathParams = lodash_zipobject_1.default(paramKeys, paramsVals);
                const r = {
                    schema,
                    expressRoute,
                    openApiRoute,
                    pathParams,
                };
                r._responseSchema = _schema;
                return r;
            }
        }
        return null;
    }
}
exports.applyOpenApiMetadata = applyOpenApiMetadata;
//# sourceMappingURL=openapi.metadata.js.map